#include <Wt/WApplication>
#include <Wt/WBreak>
#include <Wt/WCircleArea>
#include <Wt/WContainerWidget>
#include <Wt/WImage>
#include <Wt/WPolygonArea>
#include <Wt/WRectArea>
#include <Wt/WText>

using namespace Wt;

class HelloApplication : public WApplication
{
public:
  HelloApplication(const WEnvironment& env);

  enum e_what{
      TREE,
      TITLE,
      PERSON
  };

private:
  WImage *m_image;
  WText *m_out;

  void clicked(e_what what);
};

HelloApplication::HelloApplication(const WEnvironment& env)
  : WApplication(env)
{
    setTitle("Image Area - onbeforeunload");                               // application title

    setConfirmCloseMessage("If you can read this something is strange except you tried to close the browser tab/window or navigated away.");


    m_image = new WImage(WLink("pics/sintel_trailer.jpg"),
				       root());
    m_image->setAlternateText("Sintel trailer");

    new Wt::WBreak(root());
    m_out = new Wt::WText("Click a area.", root());

    Wt::WCircleArea *circle = new Wt::WCircleArea(427, 149, 58);
    circle->setToolTip("tree");
    circle->setCursor(Wt::CrossCursor);
    m_image->addArea(circle);

    Wt::WRectArea *rect = new Wt::WRectArea(294, 226, 265, 41);
    rect->setToolTip("title");
    rect->setCursor(Wt::CrossCursor);
    m_image->addArea(rect);

    Wt::WPolygonArea *polygon = new Wt::WPolygonArea();
    std::vector<Wt::WPoint> points;
    points.push_back(Wt::WPoint(92,330));
    points.push_back(Wt::WPoint(66,261));
    points.push_back(Wt::WPoint(122,176));
    points.push_back(Wt::WPoint(143,33));
    points.push_back(Wt::WPoint(164,33));
    points.push_back(Wt::WPoint(157,88));
    points.push_back(Wt::WPoint(210,90));
    points.push_back(Wt::WPoint(263,264));
    points.push_back(Wt::WPoint(228,330));
    points.push_back(Wt::WPoint(92,330));
    polygon->setPoints(points);

    polygon->setToolTip("person");
    polygon->setCursor(Wt::CrossCursor);
    m_image->addArea(polygon);

    circle->clicked().connect(boost::bind(&HelloApplication::clicked, this, TREE));

    rect->clicked().connect(boost::bind(&HelloApplication::clicked, this, TITLE));

    polygon->clicked().connect(boost::bind(&HelloApplication::clicked, this, PERSON));
}

void HelloApplication::clicked(e_what what)
{
    switch( what )
    {
    case TREE:
        m_out->setText("You clicked the tree.");
        break;
    case TITLE:
        m_out->setText("You clicked the title.");
        break;
    case PERSON:
        m_out->setText("You clicked the person.");
        break;
    default:
        m_out->setText("");
    }
}

WApplication *createApplication(const WEnvironment& env)
{
  return new HelloApplication(env);
}

int main(int argc, char **argv)
{
  return WRun(argc, argv, &createApplication);
}

